/*
    Gravel - Component library for JSF
    Copyright (C) 2007  David M. Lloyd

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.jboss.gravel.data;

import org.jboss.gravel.common.handler.CollectionHandler;
import org.jboss.gravel.data.handler.EnumStringConverterHandler;
import org.jboss.gravel.data.handler.FilterHandler;
import org.jboss.gravel.data.handler.InputFileHandler;
import org.jboss.gravel.data.handler.LoadPropertiesHandler;
import org.jboss.gravel.data.handler.ReverseHandler;
import org.jboss.gravel.data.handler.SortHandler;
import org.jboss.gravel.data.handler.PagerHandler;
import org.jboss.gravel.data.renderer.InputFileRenderer;
import org.jboss.gravel.data.renderer.InputHiddenRenderer;
import org.jboss.gravel.data.ui.UIInputFile;
import org.jboss.gravel.data.ui.UIInputHidden;
import org.jboss.gravel.data.ui.UIProperties;
import org.jboss.gravel.data.ui.UIRepeat;

import com.sun.facelets.tag.AbstractTagLibrary;

import javax.faces.application.Application;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.render.RenderKit;

/**
 *
 */
public final class DataLibrary extends AbstractTagLibrary {
    public DataLibrary() {
        super("http://gravel.jboss.org/jsf/1.0/data");

        final FacesContext facesContext = FacesContext.getCurrentInstance();
        final RenderKit renderKit = facesContext.getRenderKit();
        final Application application = facesContext.getApplication();
        final ExternalContext externalContext = facesContext.getExternalContext();

        final String maxSizeParam = externalContext.getInitParameter("gravel.FileInput.MAX_SIZE");
        final long maxSize;
        if (maxSizeParam == null) {
            maxSize = 256L * 1024L;
        } else {
            maxSize = Long.parseLong(maxSizeParam);
        }

        // Add all components
        application.addComponent(UIRepeat.COMPONENT_TYPE, UIRepeat.class.getName());
        addComponent("repeat", UIRepeat.COMPONENT_TYPE, UIRepeat.RENDERER_TYPE, CollectionHandler.class);

        application.addComponent(UIInputFile.COMPONENT_TYPE, UIInputFile.class.getName());
        addComponent("inputFile", UIInputFile.COMPONENT_TYPE, UIInputFile.RENDERER_TYPE, InputFileHandler.class);
        renderKit.addRenderer(UIInputFile.COMPONENT_FAMILY, UIInputFile.RENDERER_TYPE, new InputFileRenderer(maxSize));

        application.addComponent(UIInputHidden.COMPONENT_TYPE, UIInputHidden.class.getName());
        addComponent("inputHidden", UIInputHidden.COMPONENT_TYPE, UIInputHidden.RENDERER_TYPE);
        renderKit.addRenderer(UIInputHidden.COMPONENT_FAMILY, UIInputHidden.RENDERER_TYPE, new InputHiddenRenderer());

        addTagHandler("filter", FilterHandler.class);
        addTagHandler("reverse", ReverseHandler.class);
        addTagHandler("sort", SortHandler.class);
        addTagHandler("pager", PagerHandler.class);
        addTagHandler("enumStringConverter", EnumStringConverterHandler.class);

        application.addComponent(UIProperties.COMPONENT_TYPE, UIProperties.class.getName());
        addComponent("loadProperties", UIProperties.COMPONENT_TYPE, null, LoadPropertiesHandler.class);
    }
}
