package org.jboss.gravel.data.phase;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.faces.FacesException;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 */
@SuppressWarnings ({"deprecation"})
public final class GravelHttpServletRequest extends HttpServletRequestWrapper {
    private static final EmptyServletInputStream theInputStream = new EmptyServletInputStream();
    private static final BufferedReader theReader = new BufferedReader(new InputStreamReader(theInputStream));

    public GravelHttpServletRequest(final HttpServletRequest request) {
        super(request);
    }

    public ServletInputStream getInputStream() throws IOException {
        return theInputStream;
    }

    public BufferedReader getReader() throws IOException {
        return theReader;
    }

    // --------- Request parameter map management --------- //

    private Map<String, List<String>> parameterMap = hashMap();

    private static <K, V> Map<K, V> hashMap() {
        return new HashMap<K,V>();
    }

    public String getParameter(String string) {
        if (parameterMap.containsKey(string)) {
            final List<String> parameters = parameterMap.get(string);
            if (parameters == null || parameters.size() == 0) {
                return null;
            }
            final String value = parameters.get(0);
            if (log.isLoggable(Level.FINE)) {
                log.fine("Getting parameter: '" + string + "' -> '" + value + "'");
            }
            return value;
        } else {
            return null;
        }
    }

    public Enumeration getParameterNames() {
        return new ParameterNameEnumeration();
    }

    public String[] getParameterValues(String string) {
        if (parameterMap.containsKey(string)) {
            final List<String> list = parameterMap.get(string);
            String[] values = new String[list.size()];
            return list.toArray(values);
        } else {
            return null;
        }
    }

    public Map getParameterMap() {
        return new ParameterMap();
    }

    public void addParameter(String name, String value) {
        final List<String> parameters;
        if (parameterMap.containsKey(name)) {
            parameters = parameterMap.get(name);
        } else {
            parameters = new ArrayList<String>();
            parameterMap.put(name, parameters);
        }
        if (log.isLoggable(Level.FINE)) {
            log.fine("Adding parameter: '" + name + "' -> '" + value + "'");
        }
        parameters.add(value);
    }

    public void parseQueryString() {
        final String queryString = getQueryString();
        if (queryString == null) return;
        final int queryStringLength = queryString.length();
        int nameStart = 0;
        int valueStart = 0;
        boolean gotName = false;
        boolean gotChar = false;
        String name = null;
        try {
            for (int i = 0; i < queryStringLength; i++) {
                if (queryString.charAt(i) == '=') {
                    gotName = true;
                    valueStart = i + 1;
                    name = URLDecoder.decode(queryString.substring(nameStart, i), "UTF-8");
                } else if (queryString.charAt(i) == '&') {
                    if (gotName) {
                        // both name and value
                        addParameter(name, URLDecoder.decode(queryString.substring(valueStart, i), "UTF-8"));
                    } else if (gotChar) {
                        // just name
                        addParameter(URLDecoder.decode(queryString.substring(nameStart, i), "UTF-8"), "");
                    }
                    gotName = false;
                    gotChar = false;
                    nameStart = i + 1;
                } else {
                    gotChar = true;
                }
            }
            if (gotName) {
                // both name and value
                addParameter(name, URLDecoder.decode(queryString.substring(valueStart), "UTF-8"));
            } else if (gotChar) {
                // just name
                addParameter(name, "");
            }
        } catch (UnsupportedEncodingException e) {
            // uuhhh
            throw new FacesException("UTF-8 somehow not supported", e);
        }
    }

    private final class ParameterNameEnumeration implements Enumeration<String> {
        private final Iterator<String> iterator = parameterMap.keySet().iterator();

        public boolean hasMoreElements() {
            return iterator.hasNext();
        }

        public String nextElement() {
            return iterator.next();
        }
    }

    private final class ParameterMap implements Map<String, String[]> {

        public int size() {
            return parameterMap.size();
        }

        public boolean isEmpty() {
            return parameterMap.isEmpty();
        }

        public boolean containsKey(final Object key) {
            return parameterMap.containsKey(key);
        }

        public boolean containsValue(final Object value) {
            throw new UnsupportedOperationException("containsValue not allowed");
        }

        public String[] get(final Object key) {
            return key == null ? null : getParameterValues((String) key);
        }

        public String[] put(final String key, final String[] value) {
            throw new UnsupportedOperationException("put not allowed");
        }

        public String[] remove(final Object key) {
            throw new UnsupportedOperationException("remove not allowed");
        }

        public void putAll(final Map<? extends String, ? extends String[]> m) {
            throw new UnsupportedOperationException("putAll not allowed");
        }

        public void clear() {
            throw new UnsupportedOperationException("clear not allowed");
        }

        public Set<String> keySet() {
            return Collections.unmodifiableSet(parameterMap.keySet());
        }

        public Collection<String[]> values() {
            List<String[]> list = new ArrayList<String[]>(parameterMap.size());
            for (List<String> valueList : parameterMap.values()) {
                String[] values = new String[valueList.size()];
                list.add(valueList.toArray(values));
            }
            return Collections.unmodifiableCollection(list);
        }

        public Set<Map.Entry<String, String[]>> entrySet() {
            Set<Map.Entry<String, String[]>> entrySet = new HashSet<Entry<String, String[]>>(parameterMap.size());
            for (Entry<String, List<String>> entry : parameterMap.entrySet()) {
                final List<String> valueList = entry.getValue();
                String[] values = new String[valueList.size()];
                entrySet.add(new ParameterMapEntry(entry.getKey(), values));
            }
            return Collections.unmodifiableSet(entrySet);
        }
    }

    private static final class ParameterMapEntry implements Map.Entry<String, String[]> {

        private final String key;
        private final String[] value;

        private ParameterMapEntry(final String key, final String[] value) {
            this.key = key;
            this.value = value;
        }

        public String getKey() {
            return key;
        }

        public String[] getValue() {
            return value;
        }

        public String[] setValue(final String[] value) {
            throw new UnsupportedOperationException("setValue not allowed");
        }

        public boolean equals(final Object o) {
            if (! (o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry e2 = (Map.Entry) o;
            return (key.equals(e2.getKey()) && value.equals(e2.getValue()));
        }

        public int hashCode() {
            return key.hashCode() ^ value.hashCode();
        }
    }

    private static final Logger log = Logger.getLogger("org.jboss.gravel.data.phase.GravelHttpServletRequest");
}
