package org.jboss.gravel.data.phase;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.List;
import org.jboss.gravel.common.phase.GravelPhaseListener;

import javax.faces.context.FacesContext;

import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 */
public final class InputFileMimePartHandler implements MimePartHandler<FacesContext> {

    private final long defaultMaxSize;
    private final long defaultMaxMemorySize;

    public InputFileMimePartHandler(final long defaultMaxSize, final long defaultMaxMemorySize) {
        this.defaultMaxSize = defaultMaxSize;
        this.defaultMaxMemorySize = defaultMaxMemorySize;
    }

    public void handlePart(final FacesContext facesContext, final String name, final String filename,
               final String contentType, final String contentEncoding, final String disposition,
               final InputStream data) throws IOException {

        final Object requestObject = facesContext.getExternalContext().getRequest();

        if (requestObject instanceof GravelHttpServletRequest) {
            GravelHttpServletRequest request = (GravelHttpServletRequest) requestObject;
            if (filename != null && filename.length() > 0) {
                request.addParameter(name, filename);
                boolean tempFileOk = false;
                final File tempFile = File.createTempFile("gravel", null);
                try {
                    tempFile.deleteOnExit();
                    final List<Runnable> cleaners = GravelPhaseListener.getRequestCleaners(facesContext);
                    cleaners.add(GravelPhaseListener.fileCleaner(tempFile));
                    final FileOutputStream outputStream = new FileOutputStream(tempFile);
                    boolean outputStreamOk = false;
                    try {
                        byte[] buffer = new byte[4096];
                        int res;
                        do {
                            res = data.read(buffer);
                            if (res > 0) {
                                outputStream.write(buffer, 0, res);
                            }
                        } while (res > -1);
                        outputStream.flush();
                        outputStream.close();
                        outputStreamOk = true;
                    } finally {
                        if (! outputStreamOk) {
                            try {
                                outputStream.close();
                            } catch (IOException ex) {
                                log.log(Level.FINE, "Error closing resource on error path", ex);
                            }
                        }
                    }
                    tempFileOk = true;
                } finally {
                    if (! tempFileOk) {
                        tempFile.delete();
                    }
                }
                request.setAttribute("gravel.private.file." + name, tempFile);
            } else {
                request.addParameter(name, getStringValue(data, defaultMaxSize, defaultMaxMemorySize));
            }
        }
    }

    private String getStringValue(final InputStream data, final long maxSize, final long maxMemorySize) throws IOException {
        long limit = Math.min(maxMemorySize, maxSize);
        int actualMemSize = limit > (long) Integer.MAX_VALUE ? Integer.MAX_VALUE : (int) limit;
        char[] chars = new char[actualMemSize];
        String output = null;
        final Reader reader = new InputStreamReader(data);
        boolean readerOk = false;
        try {
            int idx = 0;
            while (idx < actualMemSize) {
                final int cnt = reader.read(chars, idx, actualMemSize - idx);
                if (cnt == -1) {
                    output = new String(chars, 0, idx);
                    break;
                }
                idx += cnt;
            }
            if (output == null) {
                output = new String(chars);
            }
            reader.close();
            readerOk = true;
        } finally {
            if (! readerOk) {
                try {
                    reader.close();
                } catch (IOException ex) {
                    log.log(Level.FINE, "Error closing resource on error path", ex);
                }
            }
        }
        return output;
    }

    private static final Logger log = Logger.getLogger("org.jboss.gravel.data.phase.InputFileMimePartHandler");
}
