package org.jboss.gravel.data.phase;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.faces.FacesException;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.http.HttpServletRequest;

import java.util.logging.Logger;

/**
 *
 */
public final class InputFilePhaseListener implements PhaseListener {
    private static final long serialVersionUID = 1L;

    private boolean initted = false;
    private long maxMemorySize;
    private long maxSize;

    private void init(final FacesContext facesContext) {
        final ExternalContext externalContext = facesContext.getExternalContext();
        final String maxMemorySizeParam = externalContext.getInitParameter("gravel.FileInput.MAX_MEMORY_SIZE");
        final String maxSizeParam = externalContext.getInitParameter("gravel.FileInput.MAX_SIZE");
        final long maxMemorySize;
        final long maxSize;
        if (maxMemorySizeParam == null) {
            maxMemorySize = 512L;
        } else {
            maxMemorySize = Long.parseLong(maxMemorySizeParam);
        }
        if (maxSizeParam == null) {
            maxSize = 256L * 1024L;
        } else {
            maxSize = Long.parseLong(maxSizeParam);
        }

        if (maxMemorySize > maxSize) {
            throw new FacesException("Maximum parameter size parameter may not exceed maximum file size parameter");
        }
        this.maxMemorySize = maxMemorySize;
        this.maxSize = maxSize;
        initted = true;
    }

    private static final Pattern multipartPattern
        = Pattern.compile("^\\s*multipart/form-data\\s*[;,]\\s*boundary\\s*=\\s*(\\S+)\\s*$");

    public void beforePhase(PhaseEvent event) {
        final FacesContext facesContext = event.getFacesContext();
        if (! initted) init(facesContext);
        if (event.getPhaseId() == PhaseId.RESTORE_VIEW) {
            final ExternalContext externalContext = facesContext.getExternalContext();
            final Object request = externalContext.getRequest();
            if (request instanceof HttpServletRequest) {
                final HttpServletRequest httpServletRequest = (HttpServletRequest) request;
                final String contentType = httpServletRequest.getContentType();
                if (contentType == null) {
                    log.fine("Not parsing request - there is no request body");
                } else {
                    final Matcher matcher = multipartPattern.matcher(contentType);
                    if (matcher.matches()) {
                        final String boundary = matcher.group(1);
                        log.fine("Got multipart/form-data, with boundary \"" + boundary + "\"");
                        try {
                            final byte[] boundaryBytes = ("--" + boundary).getBytes("US-ASCII");
                            GravelHttpServletRequest newRequest = new GravelHttpServletRequest(httpServletRequest);
                            externalContext.setRequest(newRequest);
                            newRequest.parseQueryString();
                            final MimeParser parser = new MimeParser(httpServletRequest.getInputStream(), boundaryBytes);
                            parser.parse(new InputFileMimePartHandler(maxSize, maxMemorySize), facesContext);
                        } catch (UnsupportedEncodingException e) {
                            throw new FacesException("US-ASCII not supported, this should never happen");
                        } catch (IOException e) {
                            throw new FacesException("Error reading input stream: " + e.getMessage());
                        }
                    } else {
                        log.fine("Not parsing request - it is not a known request type (" + contentType + ")");
                    }
                }
            } else {
                log.fine("Not parsing request - it is not an HttpServletRequest");
            }
        }
    }

    public void afterPhase(PhaseEvent event) {
        // nothing
    }

    public PhaseId getPhaseId() {
        return PhaseId.ANY_PHASE;
    }

    private static final Logger log = Logger.getLogger("org.jboss.gravel.data.phase.FileInputPhaseListener");
}
