package org.jboss.gravel.data.ui;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import org.jboss.gravel.common.annotation.TldAttribute;
import org.jboss.gravel.common.annotation.TldTag;
import org.jboss.gravel.common.ui.HasHtmlBasicAttributes;
import org.jboss.gravel.common.ui.HasHtmlSelectEventsAttributes;
import org.jboss.gravel.common.ui.SizeLimitedInput;
import org.jboss.gravel.common.ui.UIGravelBase;

import javax.faces.context.FacesContext;

/**
 *
 */
@TldTag (
    name = "inputFile",
    description = "Upload a file from the client to the server.  <b>Note:</b> this element must be enclosed " +
        "within a form whose <code>enctype</code> is set to <code>multipart/form-data</code> in order for it " +
        "to work properly.  Also, be aware that using a target type other than <code>stream</code> or <code>reader</code> " +
        "on large files can be severely detrimental to performance."
)
public final class UIInputFile extends UIGravelBase implements HasHtmlBasicAttributes, HasHtmlSelectEventsAttributes, SizeLimitedInput {
    private static final long serialVersionUID = 1L;

    public static final String COMPONENT_TYPE = "gravel.data.InputFile";
    public static final String RENDERER_TYPE = "gravel.data.InputFile";
    public static final String COMPONENT_FAMILY = "gravel.data";

    public UIInputFile() {
        setRendererType(RENDERER_TYPE);
    }

    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    public static enum Type {
        STREAM,
        READER,
        STRING,
        BYTES,
        CHARS
    }

    private static final Map<String, Type> typeMap = hashMap();

    static {
        typeMap.put("stream", UIInputFile.Type.STREAM);
        typeMap.put("reader", UIInputFile.Type.READER);
        typeMap.put("string", UIInputFile.Type.STRING);
        typeMap.put("bytes", UIInputFile.Type.BYTES);
        typeMap.put("chars", UIInputFile.Type.CHARS);
    }

    private static <K, V> Map<K, V> hashMap() {
        return new HashMap<K, V>();
    }

    // ================ Properties ================

    // htmlCore properties

    private String styleClass;
    private String style;
    private String title;

    public String getStyleClass() {
        return getAttributeValue("styleClass", styleClass);
    }

    public void setStyleClass(final String styleClass) {
        this.styleClass = styleClass;
    }

    public String getStyle() {
        return getAttributeValue("style", style);
    }

    public void setStyle(final String style) {
        this.style = style;
    }

    public String getTitle() {
        return getAttributeValue("title", title);
    }

    public void setTitle(final String title) {
        this.title = title;
    }

    // htmlI18n properties

    private String lang;
    private String dir;

    public String getLang() {
        return getAttributeValue("lang", lang);
    }

    public void setLang(final String lang) {
        this.lang = lang;
    }

    public String getDir() {
        return getAttributeValue("dir", dir);
    }

    public void setDir(final String dir) {
        this.dir = dir;
    }

    // htmlEvents properties

    private String onclick;
    private String ondblclick;
    private String onmousedown;
    private String onmouseup;
    private String onmouseover;
    private String onmousemove;
    private String onmouseout;
    private String onkeypress;
    private String onkeydown;
    private String onkeyup;
    private String onload;
    private String onunload;

    public String getOnclick() {
        return getAttributeValue("onclick", onclick);
    }

    public void setOnclick(final String onclick) {
        this.onclick = onclick;
    }

    public String getOndblclick() {
        return getAttributeValue("ondblclick", ondblclick);
    }

    public void setOndblclick(final String ondblclick) {
        this.ondblclick = ondblclick;
    }

    public String getOnmousedown() {
        return getAttributeValue("onmousedown", onmousedown);
    }

    public void setOnmousedown(final String onmousedown) {
        this.onmousedown = onmousedown;
    }

    public String getOnmouseup() {
        return getAttributeValue("onmouseup", onmouseup);
    }

    public void setOnmouseup(final String onmouseup) {
        this.onmouseup = onmouseup;
    }

    public String getOnmouseover() {
        return getAttributeValue("onmouseover", onmouseover);
    }

    public void setOnmouseover(final String onmouseover) {
        this.onmouseover = onmouseover;
    }

    public String getOnmousemove() {
        return getAttributeValue("onmousemove", onmousemove);
    }

    public void setOnmousemove(final String onmousemove) {
        this.onmousemove = onmousemove;
    }

    public String getOnmouseout() {
        return getAttributeValue("onmouseout", onmouseout);
    }

    public void setOnmouseout(final String onmouseout) {
        this.onmouseout = onmouseout;
    }

    public String getOnkeypress() {
        return getAttributeValue("onkeypress", onkeypress);
    }

    public void setOnkeypress(final String onkeypress) {
        this.onkeypress = onkeypress;
    }

    public String getOnkeydown() {
        return getAttributeValue("onkeydown", onkeydown);
    }

    public void setOnkeydown(final String onkeydown) {
        this.onkeydown = onkeydown;
    }

    public String getOnkeyup() {
        return getAttributeValue("onkeyup", onkeyup);
    }

    public void setOnkeyup(final String onkeyup) {
        this.onkeyup = onkeyup;
    }

    public String getOnload() {
        return getAttributeValue("onload", onload);
    }

    public void setOnload(final String onload) {
        this.onload = onload;
    }

    public String getOnunload() {
        return getAttributeValue("onunload", onunload);
    }

    public void setOnunload(final String onunload) {
        this.onunload = onunload;
    }

    // input events properties

    private String onselect;
    private String onchange;
    private String onfocus;
    private String onblur;

    public String getOnselect() {
        return getAttributeValue("onselect", onselect);
    }

    public void setOnselect(final String onselect) {
        this.onselect = onselect;
    }

    public String getOnchange() {
        return getAttributeValue("onchange", onchange);
    }

    public void setOnchange(final String onchange) {
        this.onchange = onchange;
    }

    public String getOnfocus() {
        return getAttributeValue("onfocus", onfocus);
    }

    public void setOnfocus(final String onfocus) {
        this.onfocus = onfocus;
    }

    public String getOnblur() {
        return getAttributeValue("onblur", onblur);
    }

    public void setOnblur(final String onblur) {
        this.onblur = onblur;
    }

    // specific

    private String fileName;
    private boolean disabled;
    private boolean disabledSet;
    private boolean readonly;
    private boolean readonlySet;
    private String size;
    private String alt;
    private int maxlength;
    private boolean maxlengthSet;
    private int tabindex;
    private boolean tabindexSet;
    private String accesskey;
    private String accept;

    @TldAttribute(
        description = "A file name to prepopulate this form field with."
    )
    public String getFileName() {
        return getAttributeValue("fileName", fileName);
    }

    public void setFileName(final String fileName) {
        this.fileName = fileName;
    }

    @TldAttribute(
        description = "A flag that indicates whether this field is disabled."
    )
    public boolean isDisabled() {
        return getAttributeValue("disabled", disabled, disabledSet, false);
    }

    public void setDisabled(final boolean disabled) {
        this.disabled = disabled;
    }

    @TldAttribute(
        description = "A flag that indicates whether this field may be changed."
    )
    public boolean isReadonly() {
        return getAttributeValue("readonly", disabled, disabledSet, false);
    }

    public void setReadonly(final boolean readonly) {
        this.readonly = readonly;
    }

    @TldAttribute(
        description = "This attribute tells the user agent the initial width of the control, in pixels."
    )
    public String getSize() {
        return getAttributeValue("size", size);
    }

    public void setSize(final String size) {
        this.size = size;
    }

    @TldAttribute(
        description = "Alternate text to display for user agents that cannot render this type of form field."
    )
    public String getAlt() {
        return getAttributeValue("alt", alt);
    }

    public void setAlt(final String alt) {
        this.alt = alt;
    }

    public int getMaxlength() {
        return getAttributeValue("maxlength", maxlength, maxlengthSet, -1);
    }

    public void setMaxlength(final int maxlength) {
        this.maxlength = maxlength;
    }

    @TldAttribute(
        description = "This attribute specifies the position of the current element in the tabbing order for " +
            "the current document. This value must be a number between 0 and 32767."
    )
    public int getTabindex() {
        return getAttributeValue("tabindex", tabindex, tabindexSet, -1);
    }

    public void setTabindex(final int tabindex) {
        this.tabindex = tabindex;
    }

    @TldAttribute(
        description = "This attribute assigns an access key to an element. An access key is a single character from " +
            "the document character set. Note: Authors should consider the input method of the expected reader when " +
            "specifying an accesskey."
    )
    public String getAccesskey() {
        return getAttributeValue("accesskey", accesskey);
    }

    public void setAccesskey(final String accesskey) {
        this.accesskey = accesskey;
    }

    @TldAttribute(
        description = "This attribute specifies a comma-separated list of content types that a server processing " +
            "this form will handle correctly. User agents may use this information to filter out non-conforming " +
            "files when prompting a user to select files to be sent to the server."
    )
    public String getAccept() {
        return getAttributeValue("accept", accept);
    }

    public void setAccept(final String accept) {
        this.accept = accept;
    }

    // fileUpload properties

    private String targetType;
    private String targetCharset;
    private long maxMemorySize;
    private boolean maxMemorySizeSet;
    private long maxSize;
    private boolean maxSizeSet;

    public String getTargetType() {
        return getAttributeValue("targetType", targetType, "stream");
    }

    public Type getTargetTypeEnum() {
        return typeMap.get(getTargetType());
    }

    public void setTargetType(final String targetType) {
        this.targetType = targetType;
    }

    public String getTargetCharset() {
        return getAttributeValue("targetCharset", targetCharset);
    }

    public void setTargetCharset(final String targetCharset) {
        this.targetCharset = targetCharset;
    }

    public long getMaxMemorySize() {
        return getAttributeValue("maxMemorySize", maxMemorySize, maxMemorySizeSet);
    }

    public void setMaxMemorySize(final long maxMemorySize) {
        this.maxMemorySize = maxMemorySize;
        maxMemorySizeSet = true;
    }

    public long getMaxSize() {
        return getAttributeValue("maxSize", maxSize, maxSizeSet);
    }

    public void setMaxSize(final long maxSize) {
        this.maxSize = maxSize;
        maxSizeSet = true;
    }

    public void updateTarget(Object targetValue) {
        getValueExpression("target").setValue(getFacesContext().getELContext(), targetValue);
    }

    // ================ State management ================

    private State state;

    public Object saveState(final FacesContext context) {
        if (state == null) {
            state = new State();
        }
        state.superState = super.saveState(context);
        state.styleClass = styleClass;
        state.style = style;
        state.title = title;
        state.lang = lang;
        state.dir = dir;
        state.onclick = onclick;
        state.ondblclick = ondblclick;
        state.onmousedown = onmousedown;
        state.onmouseup = onmouseup;
        state.onmouseover = onmouseover;
        state.onmousemove = onmousemove;
        state.onmouseout = onmouseout;
        state.onkeypress = onkeypress;
        state.onkeydown = onkeydown;
        state.onkeyup = onkeyup;
        state.onload = onload;
        state.onunload = onunload;
        state.onselect = onselect;
        state.onchange = onchange;
        state.onfocus = onfocus;
        state.onblur = onblur;
        state.fileName = fileName;
        state.disabled = disabled;
        state.disabledSet = disabledSet;
        state.readonly = readonly;
        state.readonlySet = readonlySet;
        state.size = size;
        state.alt = alt;
        state.maxlength = maxlength;
        state.maxlengthSet = maxlengthSet;
        state.tabindex = tabindex;
        state.tabindexSet = tabindexSet;
        state.accesskey = accesskey;
        state.accept = accept;
        state.targetType = targetType;
        state.targetCharset = targetCharset;
        state.maxMemorySize = maxMemorySize;
        state.maxMemorySizeSet = maxMemorySizeSet;
        state.maxSize = maxSize;
        state.maxSizeSet = maxSizeSet;

        return state;
    }

    public void restoreState(final FacesContext context, final Object object) {
        state = (State) object;
        styleClass = state.styleClass;
        style = state.style;
        title = state.title;
        lang = state.lang;
        dir = state.dir;
        onclick = state.onclick;
        ondblclick = state.ondblclick;
        onmousedown = state.onmousedown;
        onmouseup = state.onmouseup;
        onmouseover = state.onmouseover;
        onmousemove = state.onmousemove;
        onmouseout = state.onmouseout;
        onkeypress = state.onkeypress;
        onkeydown = state.onkeydown;
        onkeyup = state.onkeyup;
        onload = state.onload;
        onunload = state.onunload;
        onselect = state.onselect;
        onchange = state.onchange;
        onfocus = state.onfocus;
        onblur = state.onblur;
        fileName = state.fileName;
        disabled = state.disabled;
        disabledSet = state.disabledSet;
        readonly = state.readonly;
        readonlySet = state.readonlySet;
        size = state.size;
        alt = state.alt;
        maxlength = state.maxlength;
        maxlengthSet = state.maxlengthSet;
        tabindex = state.tabindex;
        tabindexSet = state.tabindexSet;
        accesskey = state.accesskey;
        accept = state.accept;
        targetType = state.targetType;
        targetCharset = state.targetCharset;
        maxMemorySize = state.maxMemorySize;
        maxMemorySizeSet = state.maxMemorySizeSet;
        maxSize = state.maxSize;
        maxSizeSet = state.maxSizeSet;

        super.restoreState(context, state.superState);
    }

    public static final class State implements Serializable {
        private static final long serialVersionUID = 1L;

        private Object superState;

        private String styleClass;
        private String style;
        private String title;

        private String lang;
        private String dir;

        private String onclick;
        private String ondblclick;
        private String onmousedown;
        private String onmouseup;
        private String onmouseover;
        private String onmousemove;
        private String onmouseout;
        private String onkeypress;
        private String onkeydown;
        private String onkeyup;
        private String onload;
        private String onunload;

        private String onselect;
        private String onchange;
        private String onfocus;
        private String onblur;

        private String fileName;
        private boolean disabled;
        private boolean disabledSet;
        private boolean readonly;
        private boolean readonlySet;
        private String size;
        private String alt;
        private int maxlength;
        private boolean maxlengthSet;
        private int tabindex;
        private boolean tabindexSet;
        private String accesskey;
        private String accept;

        private String targetType;
        private String targetCharset;
        private long maxMemorySize;
        private boolean maxMemorySizeSet;
        private long maxSize;
        private boolean maxSizeSet;
    }
}