package org.jboss.gravel.data.ui;

import org.jboss.gravel.common.annotation.TldAttribute;
import org.jboss.gravel.common.annotation.TldTag;
import org.jboss.gravel.common.ui.HasJsfCoreAttributes;
import org.jboss.gravel.common.ui.UIGravelBase;

import javax.faces.context.FacesContext;
import javax.faces.component.EditableValueHolder;
import javax.faces.convert.Converter;
import javax.faces.el.MethodBinding;
import javax.faces.validator.Validator;
import javax.faces.event.ValueChangeListener;
import javax.el.ValueExpression;

import java.util.List;
import java.util.ArrayList;
import java.io.Serializable;

import java.util.logging.Logger;
import java.util.logging.Level;

/**
 *
 */
// TODO - Extract a better Input interface
// TODO - Add "validator" attribute once method bindings are added to gravel tools
@TldTag (
    name = "inputHidden",
    description = "Render an HTML 'input' element of type 'text'.  Specifies a separate target EL expression from " +
        "the input value.",
    attributes = {
        @TldAttribute(
            name = "value",
            description = "The current value of this component.  This value will be written out in the component on " +
                "encode.",
            deferredType = Object.class
        ),
        @TldAttribute(
            name = "target",
            description = "The target to store the read-back value.  This expression will be updated upon postback.",
            deferredType = Object.class
        )
    }
)
public final class UIInputHidden extends UIGravelBase implements HasJsfCoreAttributes, EditableValueHolder {

    public static final String COMPONENT_TYPE = "gravel.data.InputHidden";
    public static final String RENDERER_TYPE = "gravel.data.InputHidden";
    public static final String COMPONENT_FAMILY = "gravel.data";

    public void decode(FacesContext context) {
        log.fine("XXXX ZERO");
        super.decode(context);
    }

    public void processDecodes(FacesContext context) {
        log.fine("XXXX -1");
        super.processDecodes(context);
    }

    public UIInputHidden() {
        setRendererType(RENDERER_TYPE);
    }

    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    // Editable value holder methods

    public Object getSubmittedValue() {
        return null;
    }

    public void setSubmittedValue(final Object submittedValue) {
    }

    public boolean isLocalValueSet() {
        return value != null;
    }

    public void setLocalValueSet(boolean localValueSet) {
    }

    public boolean isValid() {
        return true;
    }

    public void setValid(boolean valid) {
    }

    public boolean isRequired() {
        return false;
    }

    public void setRequired(boolean required) {
    }

    public boolean isImmediate() {
        return true;
    }

    public void setImmediate(boolean immediate) {
    }

    // Don't support the deprecated stuff

    public MethodBinding getValidator() {
        return null;
    }

    public void setValidator(MethodBinding validatorBinding) {
    }

    public MethodBinding getValueChangeListener() {
        return null;
    }

    public void setValueChangeListener(MethodBinding valueChangeMethod) {
    }

    // This is the actual useful stuff

    private List<Validator> validators = new ArrayList<Validator>();

    public void addValidator(Validator validator) {
        validators.add(validator);
    }

    public Validator[] getValidators() {
        return validators.toArray(new Validator[validators.size()]);
    }

    public void removeValidator(Validator validator) {
        validators.remove(validator);
    }

    private List<ValueChangeListener> valueChangeListeners = new ArrayList<ValueChangeListener>();

    public void addValueChangeListener(ValueChangeListener listener) {
        valueChangeListeners.add(listener);
    }

    public ValueChangeListener[] getValueChangeListeners() {
        return valueChangeListeners.toArray(new ValueChangeListener[valueChangeListeners.size()]);
    }

    public void removeValueChangeListener(ValueChangeListener listener) {
        valueChangeListeners.remove(listener);
    }

    // value holder methods

    public Object getLocalValue() {
        return value;
    }

    private Object value;

    public Object getValue() {
        return getAttributeValue("value", value);
    }

    public void setValue(Object value) {
        this.value = value;
    }

    public void updateTarget(FacesContext context, Object value) {
        if (log.isLoggable(Level.FINE)) {
            log.fine("Updating target expression with value " + value);
        }
        final ValueExpression valueExpression = getValueExpression("target");
        valueExpression.setValue(context.getELContext(), value);
    }

    private Converter converter;

    public Converter getConverter() {
        return converter;
    }

    public void setConverter(Converter converter) {
        this.converter = converter;
    }

    // ================ State management ================

    private static final class State implements Serializable {

        private static final long serialVersionUID = 1L;

        private Object superState;

        private Converter converter;
        private List<ValueChangeListener> valueChangeListeners = new ArrayList<ValueChangeListener>();
        private List<Validator> validators = new ArrayList<Validator>();
    }

    private State state;

    public Object saveState(FacesContext context) {
        if (state == null) {
            state = new State();
        }
        state.superState = super.saveState(context);
        state.converter = converter;
        state.valueChangeListeners = valueChangeListeners;
        state.validators = validators;
        return state;
    }

    public void restoreState(FacesContext context, Object stateObject) {
        state = (State) stateObject;
        converter = state.converter;
        valueChangeListeners = state.valueChangeListeners;
        validators = state.validators;
        super.restoreState(context, state.superState);
    }

    private static final Logger log = Logger.getLogger("org.jboss.gravel.data.ui.UIInputHidden");
}
