package org.jboss.gravel.navigation;

import org.jboss.gravel.navigation.action.NavActionListener;
import org.jboss.gravel.navigation.executor.NavigationExecutor;
import org.jboss.gravel.navigation.executor.StayExecutorImpl;

import javax.faces.application.NavigationHandler;
import javax.faces.component.ActionSource;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ActionListener;

import java.util.logging.Logger;
import java.util.logging.Level;

/**
 *
 */
public final class GravelNavigationHandler extends NavigationHandler {
    private static final NavigationExecutor DEFAULT_EXECUTOR = new StayExecutorImpl();

    public void handleNavigation(final FacesContext context, final String fromAction, final String actionOutcome) {
        final NavigationExecutor executor;
        final NavigationState state = (NavigationState) context.getExternalContext().getRequestMap().get(NavigationState.NAV_KEY);
        if (state == null) {
            log.fine("No navigation state in current request context.  Staying on current view.");
            executor = DEFAULT_EXECUTOR;
        } else {
            final ActionEvent actionEvent = state.getActionEvent();
            if (actionEvent == null) {
                log.fine("No actionEvent in current request context.  Staying on current view.");
                executor = DEFAULT_EXECUTOR;
            } else {
                final NavActionListener navActionListener;

                final UIComponent component = actionEvent.getComponent();
                final ActionSource actionSource = (ActionSource) component;
                final ActionListener[] actionListeners = actionSource.getActionListeners();
                if (actionListeners.length == 0 || ! (actionListeners[0] instanceof NavActionListener)) {
                    log.fine("No navigation rules defined on action source component.  Staying on current view.");
                    executor = DEFAULT_EXECUTOR;
                } else {
                    navActionListener = (NavActionListener) actionListeners[0];
                    if (navActionListener.hasRule(actionOutcome)) {
                        executor = navActionListener.getRule(actionOutcome);
                        if (log.isLoggable(Level.FINE)) {
                            log.fine("Navigation rule '" + actionOutcome + "' matched to executor " + executor);
                        }
                    } else if (navActionListener.hasRule("default")) {
                        executor = navActionListener.getRule("default");
                        if (log.isLoggable(Level.FINE)) {
                            log.fine("Navigation rule '" + actionOutcome + "' not matched; using registered 'default' executor " + executor);
                        }
                    } else {
                        executor = DEFAULT_EXECUTOR;
                        if (log.isLoggable(Level.FINE)) {
                            log.fine("Navigation rule '" + actionOutcome + "' not matched and no registered 'default' executor; defaulting to " + executor);
                        }
                    }
                }
            }
        }
        executor.execute(context);
        context.renderResponse();
    }

    private static final Logger log = Logger.getLogger("org.jboss.gravel.navigation.GravelNavigationHandler");
}
