package org.jboss.gravel.navigation.action;

import java.util.HashMap;
import java.util.Map;
import org.jboss.gravel.navigation.NavigationState;
import org.jboss.gravel.navigation.executor.NavigationExecutor;

import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ActionEvent;

import java.util.logging.Logger;

/**
 *
 */
public final class NavActionListenerImpl implements NavActionListener {
    private static final long serialVersionUID = 1L;

    private final Map<String,NavigationExecutor> navMap = new HashMap<String,NavigationExecutor>();

    public void addRule(String name, NavigationExecutor executor) {
        if (name == null) {
            return;
        }
        if (navMap.containsKey(name)) {
            log.warning("Navigation map already contains a rule for outcome '" + name + "'; it will be overwritten");
        }
        navMap.put(name, executor);
    }

    public void processAction(ActionEvent actionEvent) throws AbortProcessingException {
        final FacesContext context = FacesContext.getCurrentInstance();
        final Map<String, Object> requestMap = context.getExternalContext().getRequestMap();
        requestMap.put(NavigationState.NAV_KEY, new NavigationStateImpl(actionEvent));
    }

    public boolean hasRule(String name) {
        return navMap.containsKey(name);
    }

    public NavigationExecutor getRule(String name) {
        return navMap.get(name);
    }

    private static final Logger log = Logger.getLogger("org.jboss.gravel.navigation.action.NavActionListenerImpl");

    private static final class NavigationStateImpl implements NavigationState {
        private static final long serialVersionUID = 1L;

        private final ActionEvent actionEvent;

        public NavigationStateImpl(final ActionEvent actionEvent) {
            this.actionEvent = actionEvent;
        }

        public ActionEvent getActionEvent() {
            return actionEvent;
        }
    }
}
